@@ -1,6 +1,6 @@
 /* The RELP (reliable event logging protocol) core protocol library.
  *
- * Copyright 2008 by Rainer Gerhards and Adiscon GmbH.
+ * Copyright 2008-2013 by Rainer Gerhards and Adiscon GmbH.
  *
  * This file is part of librelp.
  *
@@ -34,6 +34,7 @@
 #include <stdlib.h>
 #include <sys/select.h>
 #include <string.h>
+#include <errno.h>
 #include <assert.h>
 #include "relp.h"
 #include "relpsrv.h"
@@ -211,6 +212,40 @@ relpEngineSetDbgprint(relpEngine_t *pThis, void (*dbgprint)(char *fmt, ...) __at
 }
 
 
+
+/* create a new listener to be added to the engine. This is the new-style
+ * calling convention, which permits us to set various properties before
+ * the listener is actually started. New callers should use it. Sequence
+ * is:
+ * * relpEngineListnerConstruct()
+ * * ... set properties ... (via relpSrv...() family)
+ * * relgEngineListnerConstructFinalize()
+ * rgerhards, 2013-05-14
+ */
+relpRetVal
+relpEngineListnerConstruct(relpEngine_t *pThis, relpSrv_t **ppSrv)
+{
+	relpSrv_t *pSrv;
+	ENTER_RELPFUNC;
+	RELPOBJ_assert(pThis, Engine);
+
+	CHKRet(relpSrvConstruct(&pSrv, pThis));
+	CHKRet(relpSrvSetFamily(pSrv, pThis->ai_family));
+	*ppSrv = pSrv;
+finalize_it:
+	LEAVE_RELPFUNC;
+}
+relpRetVal
+relpEngineListnerConstructFinalize(relpEngine_t *pThis, relpSrv_t *pSrv)
+{
+	ENTER_RELPFUNC;
+	RELPOBJ_assert(pThis, Engine);
+	CHKRet(relpSrvRun(pSrv));
+	CHKRet(relpEngineAddToSrvList(pThis, pSrv));
+finalize_it:
+	LEAVE_RELPFUNC;
+}
+
 /* unfortunately, we need to duplicate some functionality to support the <= 0.1.2
  * callback interface (which did not contain a user pointer) and the >= 0.1.3 
  * callback interface. I have thought a lot about doing this smarter, but there
@@ -218,7 +253,6 @@ relpEngineSetDbgprint(relpEngine_t *pThis, void (*dbgprint)(char *fmt, ...) __at
  * interface. -- rgerhards, 2008-07-08
  */
 
-
 /* a dummy for callbacks not set by the caller */
 static relpRetVal relpSrvSyslogRcvDummy2(void __attribute__((unused)) *pUsr,
 	unsigned char __attribute__((unused)) *pHostName,
@@ -240,9 +274,62 @@ relpEngineSetSyslogRcv2(relpEngine_t *pThis, relpRetVal (*pCB)(void *, unsigned
 	LEAVE_RELPFUNC;
 }
 
-/* add a relp listener to the engine. The listen port must be provided.
- * The listen port may be NULL, in which case the default port is used.
- * rgerhards, 2008-03-17
+/**
+ * Set an event handler that shall receive information when authentication
+ * has been failed.
+ *
+ * Most importantly, this permits the caller to emit error messages on
+ * failed authentication. It is also suggested to output the credentials,
+ * so that the user knows what to do (e.g. add fingerprint for newly
+ * setup peer).
+ *
+ * This handler will only be called in a mode with authenticaton.
+ * Practically, this means when TLS support is enabled.
+ * 
+ * Callback parameters:
+ *
+ * pUsr     - the user pointer set
+ * authinfo - the credentials that have been used to authenticate
+ *            the remote peer. This may be a fingerprint or something 
+ *            else, depending on authentication settings.
+ * errmsg   - error message as far as librelp is concerned
+ * errcode  - contains librelp error status that lead to the failed auth.
+ */
+relpRetVal
+relpEngineSetOnAuthErr(relpEngine_t *pThis, void (*pCB)(void*pUsr, char *authinfo, char*errmsg, relpRetVal errcode) )
+{
+	ENTER_RELPFUNC;
+	RELPOBJ_assert(pThis, Engine);
+	pThis->onAuthErr = pCB;
+	LEAVE_RELPFUNC;
+}
+
+/**
+ * Set an event handler that shall receive information when some error
+ * occured for which no special handler exists. Note that even if the
+ * special handler (e.g. AuthErr) is not set, this handler here will
+ * not be called. This permits the lib-user to set fine-grained control
+ * on which error messages it intends to handle.
+ *
+ * Callback parameters:
+ *
+ * pUsr     - the user pointer set
+ * objinfo  - some information identifying the object in error; depends
+ *            on the actual error case.
+ * errmsg   - error message as far as librelp is concerned
+ * errcode  - contains librelp error status
+ */
+relpRetVal
+relpEngineSetOnErr(relpEngine_t *pThis, void (*pCB)(void*pUsr, char *objinfo, char*errmsg, relpRetVal errcode) )
+{
+	ENTER_RELPFUNC;
+	RELPOBJ_assert(pThis, Engine);
+	pThis->onErr = pCB;
+	LEAVE_RELPFUNC;
+}
+
+/* Deprecated, use relpEngineListnerConstruct() family of functions.
+ * See there for further information.
  */
 relpRetVal
 relpEngineAddListner2(relpEngine_t *pThis, unsigned char *pLstnPort, void *pUsr)
@@ -250,26 +337,20 @@ relpEngineAddListner2(relpEngine_t *pThis, unsigned char *pLstnPort, void *pUsr)
 	relpSrv_t *pSrv;
 	ENTER_RELPFUNC;
 	RELPOBJ_assert(pThis, Engine);
-
-	CHKRet(relpSrvConstruct(&pSrv, pThis));
+	CHKRet(relpEngineListnerConstruct(pThis, &pSrv));
 	CHKRet(relpSrvSetUsrPtr(pSrv, pUsr));
 	CHKRet(relpSrvSetLstnPort(pSrv, pLstnPort));
-	CHKRet(relpSrvRun(pSrv));
-
-	/* all went well, so we can add the server to our server list */
-	CHKRet(relpEngineAddToSrvList(pThis, pSrv));
-
+	CHKRet(relpEngineListnerConstructFinalize(pThis, pSrv));
 finalize_it:
 	LEAVE_RELPFUNC;
 }
-
-
 /* a dummy for callbacks not set by the caller */
 static relpRetVal relpSrvSyslogRcvDummy(unsigned char __attribute__((unused)) *pHostName,
 	unsigned char __attribute__((unused)) *pIP, unsigned char __attribute__((unused)) *pMsg,
 	size_t __attribute__((unused)) lenMsg)
 { return RELP_RET_NOT_IMPLEMENTED;
 }
+
 /* set the syslog receive callback. If NULL is provided, it is set to the
  * not implemented dummy.
  */
@@ -284,9 +365,8 @@ relpEngineSetSyslogRcv(relpEngine_t *pThis, relpRetVal (*pCB)(unsigned char*, un
 	LEAVE_RELPFUNC;
 }
 
-/* add a relp listener to the engine. The listen port must be provided.
- * The listen port may be NULL, in which case the default port is used.
- * rgerhards, 2008-03-17
+/* Deprecated, use relpEngineListnerConstruct() family of functions.
+ * See there for further information.
  */
 relpRetVal
 relpEngineAddListner(relpEngine_t *pThis, unsigned char *pLstnPort)
@@ -294,19 +374,74 @@ relpEngineAddListner(relpEngine_t *pThis, unsigned char *pLstnPort)
 	relpSrv_t *pSrv;
 	ENTER_RELPFUNC;
 	RELPOBJ_assert(pThis, Engine);
-
-	CHKRet(relpSrvConstruct(&pSrv, pThis));
+	CHKRet(relpEngineListnerConstruct(pThis, &pSrv));
 	CHKRet(relpSrvSetLstnPort(pSrv, pLstnPort));
-	CHKRet(relpSrvRun(pSrv));
-
-	/* all went well, so we can add the server to our server list */
-	CHKRet(relpEngineAddToSrvList(pThis, pSrv));
-
+	CHKRet(relpEngineListnerConstructFinalize(pThis, pSrv));
 finalize_it:
 	LEAVE_RELPFUNC;
 }
 
 
+/* the setStop method sets a flag that stops the server after the next select.
+ * In order to interrupt the select(), it is suggested to send a signal. If no
+ * signal is sent, it may take rather long to stop the server (until another
+ * machine sends data).
+ */
+relpRetVal relpEngineSetStop(relpEngine_t *pThis)
+{
+	ENTER_RELPFUNC;
+	RELPOBJ_assert(pThis, Engine);
+	pThis->bStop = 1;
+	LEAVE_RELPFUNC;
+}
+
+
+/* set the socket family to use
+ */
+relpRetVal relpEngineSetFamily(relpEngine_t *pThis, int ai_family)
+{
+	ENTER_RELPFUNC;
+	RELPOBJ_assert(pThis, Engine);
+	pThis->ai_family = ai_family;
+	LEAVE_RELPFUNC;
+}
+
+/* helper for relpEngineRun; receives data when it is time to
+ * do so. Includes all housekeeping, like closing the session.
+ */
+static inline void
+doRecv(relpEngine_t *pThis, relpEngSessLst_t *pSessEtry, int sock)
+{
+	relpRetVal localRet;
+	localRet = relpSessRcvData(pSessEtry->pSess); /* errors are handled there */
+	/* if we had an error during processing, we must shut down the session. This
+	 * is part of the protocol specification: errors are recovered by aborting the
+	 * session, which may eventually be followed by a new connect.
+	 */
+	if(localRet != RELP_RET_OK) {
+		pThis->dbgprint("relp session %d iRet %d, tearing it down\n",
+				sock, localRet);
+		relpEngineDelSess(pThis, pSessEtry);
+	}
+}
+/* helper for relpEngineRun; sends session data when it is time
+ * to send. Includes all housekeeping, like closing the session.
+ */
+static inline void
+doSend(relpEngine_t *pThis, relpEngSessLst_t *pSessEtry, int sock)
+{
+	relpRetVal localRet;
+	localRet = relpSessSndData(pSessEtry->pSess); /* errors are handled there */
+	/* if we had an error during processing, we must shut down the session. This
+	 * is part of the protocol specification: errors are recovered by aborting the
+	 * session, which may eventually be followed by a new connect.
+	 */
+	if(localRet != RELP_RET_OK) {
+		pThis->dbgprint("relp session %d iRet %d during send, tearing it down\n",
+				sock, localRet);
+		relpEngineDelSess(pThis, pSessEtry);
+	}
+}
 /* The "Run" method starts the relp engine. Most importantly, this means the engine begins
  * to read and write data to its peers. This method must be called on its own thread as it
  * will not return until the engine is finished. Note that the engine itself may (or may
@@ -329,6 +464,7 @@ relpEngineRun(relpEngine_t *pThis)
 	relpEngSessLst_t *pSessEtry;
 	relpEngSessLst_t *pSessEtryNext;
 	relpSess_t *pNewSess;
+	relpTcp_t *pTcp;
 	relpRetVal localRet;
 	int iSocks;
 	int sock;
@@ -340,8 +476,8 @@ relpEngineRun(relpEngine_t *pThis)
 	ENTER_RELPFUNC;
 	RELPOBJ_assert(pThis, Engine);
 
-	/* this is an endless loop - TODO: decide how to terminate */
-	while(1) {
+	pThis->bStop = 0;
+	while(!relpEngineShouldStop(pThis)) {
 	        maxfds = 0;
 	        FD_ZERO(&readfds);
 	        FD_ZERO(&writefds);
@@ -358,10 +494,19 @@ relpEngineRun(relpEngine_t *pThis)
 		/* Add all sessions for reception and sending (they all have just one socket) */
 		for(pSessEtry = pThis->pSessLstRoot ; pSessEtry != NULL ; pSessEtry = pSessEtry->pNext) {
 			sock = relpSessGetSock(pSessEtry->pSess);
-			FD_SET(sock, &readfds);
-			/* now check if a send request is outstanding and, if so, add it */
-			if(!relpSendqIsEmpty(pSessEtry->pSess->pSendq)) {
-				FD_SET(sock, &writefds);
+			if(relpSessTcpRequiresRtry(pSessEtry->pSess)) {
+				pThis->dbgprint("***<librelp> retry op requested for sock %d\n", sock);
+				if(relpTcpGetRtryDirection(pSessEtry->pSess->pTcp) == 0) {
+					FD_SET(sock, &readfds);
+				} else {
+					FD_SET(sock, &writefds);
+				}
+			} else {
+				FD_SET(sock, &readfds);
+				/* now check if a send request is outstanding and, if so, add it */
+				if(!relpSendqIsEmpty(pSessEtry->pSess->pSendq)) {
+					FD_SET(sock, &writefds);
+				}
 			}
 			if(sock > maxfds) maxfds = sock;
 		}
@@ -376,17 +521,28 @@ relpEngineRun(relpEngine_t *pThis)
 		}
 
 		/* wait for io to become ready */
+		if(relpEngineShouldStop(pThis)) break;
 		nfds = select(maxfds+1, (fd_set *) &readfds, &writefds, NULL, NULL);
-pThis->dbgprint("relp select returns, nfds %d\n", nfds);
+		pThis->dbgprint("relp select returns, nfds %d\n", nfds);
+		if(relpEngineShouldStop(pThis)) break;
+
+		if(nfds == -1) {
+			if(errno == EINTR) {
+				pThis->dbgprint("relp select interrupted\n");
+			} else {
+				pThis->dbgprint("relp select returned error %d\n", errno);
+			}
+			continue;
+		}
 	
 		/* and then start again with the servers (new connection request) */
-		for(pSrvEtry = pThis->pSrvLstRoot ; pSrvEtry != NULL ; pSrvEtry = pSrvEtry->pNext) {
-			for(iSocks = 1 ; iSocks <= relpSrvGetNumLstnSocks(pSrvEtry->pSrv) ; ++iSocks) {
+		for(pSrvEtry = pThis->pSrvLstRoot ; nfds && pSrvEtry != NULL ; pSrvEtry = pSrvEtry->pNext) {
+			for(iSocks = 1 ; nfds && iSocks <= relpSrvGetNumLstnSocks(pSrvEtry->pSrv) ; ++iSocks) {
+				if(relpEngineShouldStop(pThis)) break;
 				sock = relpSrvGetLstnSock(pSrvEtry->pSrv, iSocks);
 				if(FD_ISSET(sock, &readfds)) {
 					pThis->dbgprint("new connect on RELP socket #%d\n", sock);
 					localRet = relpSessAcceptAndConstruct(&pNewSess, pSrvEtry->pSrv, sock);
-pThis->dbgprint("relp accept session returns, iRet %d\n", localRet);
 					if(localRet == RELP_RET_OK) {
 						localRet = relpEngineAddToSess(pThis, pNewSess);
 					}
@@ -397,34 +553,36 @@ pThis->dbgprint("relp accept session returns, iRet %d\n", localRet);
 		}
 
 		/* now check if we have some action waiting for sessions */
-		for(pSessEtry = pThis->pSessLstRoot ; pSessEtry != NULL ; ) {
+		for(pSessEtry = pThis->pSessLstRoot ; nfds && pSessEtry != NULL ; ) {
+			if(relpEngineShouldStop(pThis)) break;
 			pSessEtryNext = pSessEtry->pNext; /* we need to cache this as we may delete the entry! */
 			sock = relpSessGetSock(pSessEtry->pSess);
-			/* read data waiting? */
-			if(FD_ISSET(sock, &readfds)) {
-				localRet = relpSessRcvData(pSessEtry->pSess); /* errors are handled there */
-				/* if we had an error during processing, we must shut down the session. This
-				 * is part of the protocol specification: errors are recovered by aborting the
-				 * session, which may eventually be followed by a new connect.
-				 */
-				if(localRet != RELP_RET_OK) {
-					pThis->dbgprint("relp session %d iRet %d, tearing it down\n",
-						        sock, localRet);
-					relpEngineDelSess(pThis, pSessEtry);
+			if(relpSessTcpRequiresRtry(pSessEtry->pSess)) {
+				pTcp = pSessEtry->pSess->pTcp;
+				if(FD_ISSET(sock, &readfds) || FD_ISSET(sock, &writefds)) {
+					if(relpTcpRtryOp(pTcp) == relpTCP_RETRY_send) {
+						doSend(pThis, pSessEtry, sock);
+						--nfds; /* indicate we have processed one */
+					} else if(relpTcpRtryOp(pTcp) == relpTCP_RETRY_recv) {
+						doRecv(pThis, pSessEtry, sock);
+						--nfds; /* indicate we have processed one */
+					} else {
+						localRet = relpTcpRtryHandshake(pSessEtry->pSess->pTcp);
+						if(localRet != RELP_RET_OK) {
+							pThis->dbgprint("relp session %d handshake iRet %d, tearing it down\n",
+									sock, localRet);
+							relpEngineDelSess(pThis, pSessEtry);
+						}
+					}
 				}
-				--nfds; /* indicate we have processed one */
-			}
-			/* are we able to write? */
-			if(FD_ISSET(sock, &writefds)) {
-				localRet = relpSessSndData(pSessEtry->pSess); /* errors are handled there */
-				/* if we had an error during processing, we must shut down the session. This
-				 * is part of the protocol specification: errors are recovered by aborting the
-				 * session, which may eventually be followed by a new connect.
-				 */
-				if(localRet != RELP_RET_OK) {
-					pThis->dbgprint("relp session %d iRet %d during send, tearing it down\n",
-						        sock, localRet);
-					relpEngineDelSess(pThis, pSessEtry);
+			} else {
+				if(FD_ISSET(sock, &readfds)) {
+					doRecv(pThis, pSessEtry, sock);
+					--nfds; /* indicate we have processed one */
+				}
+				if(FD_ISSET(sock, &writefds)) {
+					doSend(pThis, pSessEtry, sock);
+					--nfds; /* indicate we have processed one */
 				}
 			}
 
@@ -468,18 +626,20 @@ relpEngineDispatchFrame(relpEngine_t *pThis, relpSess_t *pSess, relpFrame_t *pFr
 	pThis->dbgprint("relp engine is dispatching frame with command '%s'\n", pFrame->cmd);
 
 	/* currently, we hardcode the commands. Over time, they may be dynamically 
-	 * loaded and, when so, should come from a linked list. TODO -- rgerhards, 2008-03-17
+	 * loaded and, when so, should come from a linked list.
+	 * NOTE: the command handler are sorted so that most frequently used are
+	 * at the top of the list!
 	 */
-	if(!strcmp((char*)pFrame->cmd, "open")) {
+	if(!strcmp((char*)pFrame->cmd, "syslog")) {
+		CHKRet(relpSCSyslog(pFrame, pSess));
+	} else if(!strcmp((char*)pFrame->cmd, "rsp")) {
+		CHKRet(relpSCRsp(pFrame, pSess));
+	} else if(!strcmp((char*)pFrame->cmd, "open")) {
 		CHKRet(relpSCInit(pFrame, pSess));
 	} else if(!strcmp((char*)pFrame->cmd, "close")) {
 		CHKRet(relpSCClose(pFrame, pSess));
 	} else if(!strcmp((char*)pFrame->cmd, "serverclose")) {
 		CHKRet(relpCCServerclose(pFrame, pSess));
-	} else if(!strcmp((char*)pFrame->cmd, "syslog")) {
-		CHKRet(relpSCSyslog(pFrame, pSess));
-	} else if(!strcmp((char*)pFrame->cmd, "rsp")) {
-		CHKRet(relpSCRsp(pFrame, pSess));
 	} else {
 		pThis->dbgprint("invalid or not supported relp command '%s'\n", pFrame->cmd);
 		ABORT_FINALIZE(RELP_RET_INVALID_CMD);
@@ -504,8 +664,7 @@ relpEngineCltConstruct(relpEngine_t *pThis, relpClt_t **ppClt)
 	assert(ppClt != NULL);
 
 	CHKRet(relpCltConstruct(ppClt, pThis));
-
-	pThis->dbgprint("relp engine create a new client (%p)\n", *ppClt);
+	pThis->dbgprint("relp engine created new client %p\n", *ppClt);
 
 finalize_it:
 	LEAVE_RELPFUNC;
@@ -524,8 +683,7 @@ relpEngineCltDestruct(relpEngine_t *pThis, relpClt_t **ppClt)
 	assert(ppClt != NULL);
 	RELPOBJ_assert(*ppClt, Clt);
 
-	pThis->dbgprint("relp engine destructing a client (%p)\n", *ppClt);
-
+	pThis->dbgprint("relp engine destructing client %p\n", *ppClt);
 	CHKRet(relpCltDestruct(ppClt));
 
 finalize_it:
@@ -546,11 +704,16 @@ char *relpEngineGetVersion(void)
 #	endif
 }
 
+void
+relpEngineSetShutdownImmdtPtr(relpEngine_t *pThis, int *ptr)
+{
+	if(pThis->bShutdownImmdt != ptr)
+		pThis->bShutdownImmdt = ptr;
+}
 
 /* Enable or disable a command. Note that a command can not be enabled once
  * it has been set to forbidden! There will be no error return state in this
- * case.
- * rgerhards, 2008-03-27
+ * case. -- rgerhards, 2008-03-27
  */
 relpRetVal
 relpEngineSetEnableCmd(relpEngine_t *pThis, unsigned char *pszCmd, relpCmdEnaState_t stateCmd)
@@ -559,7 +722,6 @@ relpEngineSetEnableCmd(relpEngine_t *pThis, unsigned char *pszCmd, relpCmdEnaSta
 	RELPOBJ_assert(pThis, Engine);
 	assert(pszCmd != NULL);
 
-pThis->dbgprint("ENGINE SetEnableCmd in syslog cmd state: %d\n", pThis->stateCmdSyslog);
 	if(!strcmp((char*)pszCmd, "syslog")) {
 		if(pThis->stateCmdSyslog != eRelpCmdState_Forbidden)
 			pThis->stateCmdSyslog = stateCmd;
@@ -569,7 +731,6 @@ pThis->dbgprint("ENGINE SetEnableCmd in syslog cmd state: %d\n", pThis->stateCmd
 	}
 
 finalize_it:
-pThis->dbgprint("ENGINE SetEnableCmd out syslog cmd state: %d, iRet %d\n", pThis->stateCmdSyslog, iRet);
 	LEAVE_RELPFUNC;
 }
 
